==> http-1.2.tar.gz: http-1.2/http.c <==
char http_c_rcs_id[] =
    "$Id: http.c,v 1.5 1997/11/16 12:51:39 hjp Exp $";

/*
 * http - send http requests
 *
 * Simple command line tool to send http get, head, and put requests.
 *
 * $Log: http.c,v $
 * Revision 1.5  1997/11/16 12:51:39  hjp
 * Added -H flag.
 *
 * Revision 1.4  1997/04/22 00:09:23  hjp
 * Added test for missing {-g|-b|-h|-p} option (before it would dump core
 * if none of them was given).
 * Fixed usage line.
 *
 * Revision 1.3  1997/03/08 22:49:49  hjp
 * Added man page to GNUmakefile.
 *
 * Revision 1.2  1997/03/08 22:44:25  hjp
 * Added support for proxy server.
 * Fixed serious bug with -b option (it swallowed the first 4 characters
 * of the body).
 * Added man-page.
 *
 */
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ant/string.h>
#include <ant/net.h>
#include <ant/url.h>
#include <ant/alloc.h>

#define HEADERS_MAX 100

#include "base64.h"

char *cmnd;

char *get = "GET";
char *head = "HEAD";
char *put = "PUT";


char *op;
int body;
char *basic;
char *proxy;
char *headers[HEADERS_MAX];
int nr_headers = 0;

void usage(void) {
    fprintf(stderr, "Usage: %s {-g|-h|-p|-b} [-B basic_auth] [-P proxy] url\n", cmnd);
    exit(1);
}


void do_arg(const char *url) {
    int sd; 
    char hdr[0x4000];
    char *ehdr = hdr + sizeof(hdr);
    char *p;
    int rc;
    char *p2 = NULL;
    size_t content_length = 0;
    char *send_body = NULL;
    size_t alloc = 0;
    int i;

    urlT *pu = parse_url(url);

    if (proxy) {
        if ((sd = openport(proxy, 80)) == -1) {
            fprintf(stderr, "%s: cannot connect to %s: %s\n",
                    cmnd, proxy, strerror(errno));
            exit(1);
        }
        p = nstrlimcpy(hdr, op, ehdr);
        p = nstrlimcpy(p, " ", ehdr);
        p = nstrlimcpy(p, url, ehdr);
        p = nstrlimcpy(p, " HTTP/1.0\r\n", ehdr);
    } else {
        if (!pu->location) {
            fprintf(stderr, "%s: no location in %s\n",
                    cmnd, url);
            exit(1);
        }
        if ((sd = openport(pu->location, 80)) == -1) {
            fprintf(stderr, "%s: cannot connect to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        }
        p = nstrlimcpy(hdr, op, ehdr);
        p = nstrlimcpy(p, " ", ehdr);
        if (pu->path && *pu->path) {
            p = nstrlimcpy(p, pu->path, ehdr);
        } else {
            p = nstrlimcpy(p, "/", ehdr);
        }
        if (pu->params) {
            p = nstrlimcpy(p, ";", ehdr);
            p = nstrlimcpy(p, pu->params, ehdr);
        }
        if (pu->query) {
            p = nstrlimcpy(p, "?", ehdr);
            p = nstrlimcpy(p, pu->query, ehdr);
        }
        p = nstrlimcpy(p, " HTTP/1.0\r\n", ehdr);
    }
    if (basic) {
        p = nstrlimcpy(p, "Authorization: Basic ", ehdr);
        base64_encode(p, ehdr-p, basic, strlen(basic), -1);
        p += strlen(p);
        p = nstrlimcpy(p, "\r\n", ehdr);
    }
    if (op == put) {
        int c;
        while ((c = getc(stdin)) != EOF) {
            if (content_length >= alloc) {
                alloc = alloc * 2 + 1024;
                send_body = erealloc(send_body, alloc);
            }
            send_body[content_length] = c;
            content_length++;
        }
        p = nstrlimcpy(p, "Content-length: ", ehdr);
        p += sprintf(p, "%d", content_length);
        p = nstrlimcpy(p, "\r\n", ehdr);
    }

    for (i = 0; i < nr_headers; i++) {
        p = nstrlimcpy(p, headers[i], ehdr);
        p = nstrlimcpy(p, "\r\n", ehdr);
    }

    p = nstrlimcpy(p, "\r\n", ehdr);
    assert(p < ehdr);
    if ((rc = write(sd, hdr, p - hdr)) != p - hdr) {
        if (rc == -1) {
            fprintf(stderr, "%s: cannot write to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        } else {
            fprintf(stderr, "%s: wrote only %d of %d bytes to %s\n",
                    cmnd, rc, (int)(p - hdr), pu->location);
            exit(1);
        }
    }
    if (send_body &&
        (rc = write(sd, send_body, content_length)) != content_length) {
        if (rc == -1) {
            fprintf(stderr, "%s: cannot write to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        } else {
            fprintf(stderr, "%s: wrote only %d of %d bytes to %s\n",
                    cmnd, rc, (int)content_length, pu->location);
            exit(1);
        }
    }
    p = hdr;
    memset(hdr, 0, ehdr - hdr);
    while ((rc = read(sd, p, ehdr - p - 1)) > 0) {
        p += rc;
        assert (p != ehdr);
        if ((p2 = strstr(hdr, "\r\n\r\n")) != NULL) { p2 += 4; break; }
        if ((p2 = strstr(hdr, "\n\n")) != NULL) { p2 += 2; break;  /* NCSA has got this wrong */ }
    }
    if (rc < 0) {
        fprintf(stderr, "%s: cannot read: %s\n",
                cmnd, strerror(errno));
    }
    if (!body) {
        write (1, hdr, p2 - hdr);
    }
    if (op == get) {
        if (p > p2) {
            write (1, p2, p - p2);
        }
        while ((rc = read(sd, hdr, ehdr - hdr)) > 0) {
            write (1, hdr, rc);
        }
        if (rc < 0) {
            fprintf(stderr, "%s: cannot read: %s\n",
                    cmnd, strerror(errno));
        }
    }
    close(sd);
}


int main (int argc, char **argv) {
    int c;
    int i;

    while ((c = getopt(argc, argv, "ghbpB:P:H:")) != EOF) {
        char *p;
        switch (c) {
        case 'b':
            op = get;
            body = 1;
            break;
        case 'g':
            op = get;
            break;
        case 'h':
            op = head;
            break;
        case 'p':
            op = put;
            break;
        case 'B':
            basic = strdup(optarg);
            for (p = optarg; *p; p++) *p = '?';
            break;
        case 'P':
            proxy = optarg;
            break;
        case 'H':
            if (nr_headers >= HEADERS_MAX) {
                fprintf(stderr, "%s: too many headers\n", cmnd);
                exit(1);
            }
            headers[nr_headers++] = optarg;
            break;
        case '?':
            usage();
        default:
            assert(0);
        }
    }

    if (!op) usage();

    for (i = optind; i < argc; i++) {
        do_arg(argv[i]);
    }
    return 0;
}

==> http-1.2.1.tar.gz: http-1.2.1/http.c <==
char http_c_rcs_id[] =
    "$Id: http.c,v 1.7 1998/01/14 17:52:19 hjp Exp $";

/*
 * http - send http requests
 *
 * Simple command line tool to send http get, head, and put requests.
 *
 * $Log: http.c,v $
 * Revision 1.7  1998/01/14 17:52:19  hjp
 * Added -H flag to usage.
 *
 * Revision 1.6  1998/01/14 17:48:26  hjp
 * Changed base64_encode: Now returns the number of characters it wrote
 * into dest. Before dest had to be zero-filled and at least one character
 * longer than the size passed to dest, so that the calling function could
 * look for a terminating '\0'. Not a good interface (I am still unsure
 * whether base64_encode should terminate the string it writes).
 *
 * Changed http.c to take advantage of this change (The problem was of
 * course that http.c did not zerofill the buffer before calling
 * base64_encode but relied on a '\0' immediately after the result.
 * Occasionally this didn't work.
 *
 * Revision 1.5  1997/11/16  12:51:39  hjp
 * Added -H flag.
 *
 * Revision 1.4  1997/04/22 00:09:23  hjp
 * Added test for missing {-g|-b|-h|-p} option (before it would dump core
 * if none of them was given).
 * Fixed usage line.
 *
 * Revision 1.3  1997/03/08 22:49:49  hjp
 * Added man page to GNUmakefile.
 *
 * Revision 1.2  1997/03/08 22:44:25  hjp
 * Added support for proxy server.
 * Fixed serious bug with -b option (it swallowed the first 4 characters
 * of the body).
 * Added man-page.
 *
 */
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ant/string.h>
#include <ant/net.h>
#include <ant/url.h>
#include <ant/alloc.h>

#define HEADERS_MAX 100

#include "base64.h"

char *cmnd;

char *get = "GET";
char *head = "HEAD";
char *put = "PUT";


char *op;
int body;
char *basic;
char *proxy;
char *headers[HEADERS_MAX];
int nr_headers = 0;

void usage(void) {
    fprintf(stderr, "Usage: %s {-g|-h|-p|-b} [-B basic_auth] [-P proxy] [-H header] url\n", cmnd);
    exit(1);
}


void do_arg(const char *url) {
    int sd; 
    char hdr[0x4000];
    char *ehdr = hdr + sizeof(hdr);
    char *p;
    int rc;
    char *p2 = NULL;
    size_t content_length = 0;
    char *send_body = NULL;
    size_t alloc = 0;
    int i;

    urlT *pu = parse_url(url);

    if (proxy) {
        if ((sd = openport(proxy, 80)) == -1) {
            fprintf(stderr, "%s: cannot connect to %s: %s\n",
                    cmnd, proxy, strerror(errno));
            exit(1);
        }
        p = nstrlimcpy(hdr, op, ehdr);
        p = nstrlimcpy(p, " ", ehdr);
        p = nstrlimcpy(p, url, ehdr);
        p = nstrlimcpy(p, " HTTP/1.0\r\n", ehdr);
    } else {
        if (!pu->location) {
            fprintf(stderr, "%s: no location in %s\n",
                    cmnd, url);
            exit(1);
        }
        if ((sd = openport(pu->location, 80)) == -1) {
            fprintf(stderr, "%s: cannot connect to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        }
        p = nstrlimcpy(hdr, op, ehdr);
        p = nstrlimcpy(p, " ", ehdr);
        if (pu->path && *pu->path) {
            p = nstrlimcpy(p, pu->path, ehdr);
        } else {
            p = nstrlimcpy(p, "/", ehdr);
        }
        if (pu->params) {
            p = nstrlimcpy(p, ";", ehdr);
            p = nstrlimcpy(p, pu->params, ehdr);
        }
        if (pu->query) {
            p = nstrlimcpy(p, "?", ehdr);
            p = nstrlimcpy(p, pu->query, ehdr);
        }
        p = nstrlimcpy(p, " HTTP/1.0\r\n", ehdr);
    }
    if (basic) {
        size_t n;
        p = nstrlimcpy(p, "Authorization: Basic ", ehdr);
        n = base64_encode(p, ehdr-p, basic, strlen(basic), -1);
        p += n;
        p = nstrlimcpy(p, "\r\n", ehdr);
    }
    if (op == put) {
        int c;
        while ((c = getc(stdin)) != EOF) {
            if (content_length >= alloc) {
                alloc = alloc * 2 + 1024;
                send_body = erealloc(send_body, alloc);
            }
            send_body[content_length] = c;
            content_length++;
        }
        p = nstrlimcpy(p, "Content-length: ", ehdr);
        p += sprintf(p, "%d", content_length);
        p = nstrlimcpy(p, "\r\n", ehdr);
    }

    for (i = 0; i < nr_headers; i++) {
        p = nstrlimcpy(p, headers[i], ehdr);
        p = nstrlimcpy(p, "\r\n", ehdr);
    }

    p = nstrlimcpy(p, "\r\n", ehdr);
    assert(p < ehdr);
    if ((rc = write(sd, hdr, p - hdr)) != p - hdr) {
        if (rc == -1) {
            fprintf(stderr, "%s: cannot write to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        } else {
            fprintf(stderr, "%s: wrote only %d of %d bytes to %s\n",
                    cmnd, rc, (int)(p - hdr), pu->location);
            exit(1);
        }
    }
    if (send_body &&
        (rc = write(sd, send_body, content_length)) != content_length) {
        if (rc == -1) {
            fprintf(stderr, "%s: cannot write to %s: %s\n",
                    cmnd, pu->location, strerror(errno));
            exit(1);
        } else {
            fprintf(stderr, "%s: wrote only %d of %d bytes to %s\n",
                    cmnd, rc, (int)content_length, pu->location);
            exit(1);
        }
    }
    p = hdr;
    memset(hdr, 0, ehdr - hdr);
    while ((rc = read(sd, p, ehdr - p - 1)) > 0) {
        p += rc;
        assert (p != ehdr);
        if ((p2 = strstr(hdr, "\r\n\r\n")) != NULL) { p2 += 4; break; }
        if ((p2 = strstr(hdr, "\n\n")) != NULL) { p2 += 2; break;  /* NCSA has got this wrong */ }
    }
    if (rc < 0) {
        fprintf(stderr, "%s: cannot read: %s\n",
                cmnd, strerror(errno));
    }
    if (!body) {
        write (1, hdr, p2 - hdr);
    }
    if (op == get) {
        if (p > p2) {
            write (1, p2, p - p2);
        }
        while ((rc = read(sd, hdr, ehdr - hdr)) > 0) {
            write (1, hdr, rc);
        }
        if (rc < 0) {
            fprintf(stderr, "%s: cannot read: %s\n",
                    cmnd, strerror(errno));
        }
    }
    close(sd);
}


int main (int argc, char **argv) {
    int c;
    int i;

    while ((c = getopt(argc, argv, "ghbpB:P:H:")) != EOF) {
        char *p;
        switch (c) {
        case 'b':
            op = get;
            body = 1;
            break;
        case 'g':
            op = get;
            break;
        case 'h':
            op = head;
            break;
        case 'p':
            op = put;
            break;
        case 'B':
            basic = strdup(optarg);
            for (p = optarg; *p; p++) *p = '?';
            break;
        case 'P':
            proxy = optarg;
            break;
        case 'H':
            if (nr_headers >= HEADERS_MAX) {
                fprintf(stderr, "%s: too many headers\n", cmnd);
                exit(1);
            }
            headers[nr_headers++] = optarg;
            break;
        case '?':
            usage();
        default:
            assert(0);
        }
    }

    if (!op) usage();

    for (i = optind; i < argc; i++) {
        do_arg(argv[i]);
    }
    return 0;
}
